<?php
/*-----------------------------------------------------------------------------
    IP Search - A phpBB Add-On
  ----------------------------------------------------------------------------
    acp_ip_search.php
       IP Search ACP Module Script
    File Version: 4.0.1
    Begun: August 25, 2003                 Last Modified: August 2, 2007
  ----------------------------------------------------------------------------
    Copyright 2007 by Jeremy Rogers.  Please read the license.txt included
    with the phpBB Add-On listed above for full license and copyright details.
  ----------------------------------------------------------------------------
    Class and Function Quick Reference
             Names                                 Search Labels
         acp_ip_search................................[class]
            main......................................[main]
			search_ip.................................[serip]
			search_user...............................[seruser]
			define_user_lookups.......................[dulook]
			define_guest_lookups......................[dglook]
            prep......................................[prep]
            ip_output_row.............................[ipout]
            run_query.................................[query]
-----------------------------------------------------------------------------*/

/*	acp_ip_search
	[class]
       Handles all of the IP searching functions.
	   Originally ported from IP Search for phpBB 2, but modified
	   greatly to increase future expansion.
*/
class acp_ip_search
{
	var $u_action;
	var $search_results	= array();
	var $search_terms	= array();
	var $user_ids		= array();
	var $lookups		= array();
	var $guest_lookups	= array();
	var $headings		= array();
	var $found_hosts	= array();
	var $user_mode		= FALSE;
	var $results_found	= FALSE;
	var $use_like		= FALSE;
	var $lookup_hosts	= FALSE;

/*	main
	[main]
       Sets up variables to run IP Search.
*/
	function main($id, $mode)
	{
		global $template, $phpbb_root_path, $phpEx;

		$submit = ( isset($_POST['submit']) || isset($_GET['submit']) ) ? TRUE : FALSE;

		$this->tpl_name		= 'ipsearch_search';
		$this->page_title	= 'ACP_IP_SEARCH';

		$template->assign_vars(array(
			'U_IP_SEARCH'		=> $this->u_action,
			'ANONYMOUS_USER_ID'	=> ANONYMOUS,
			'U_FIND_USERNAME'	=> append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&amp;form=select_user&amp;field=username&amp;select_single=true'),
			'UA_FIND_USERNAME'	=> append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&form=select_user&field=username&select_single=true', false),
		));

		if( !$submit )
		{
			// We're done... quit to print the basic search form.
			return;
		}

		// Got a submit, do a search.
		$this->tpl_name		= 'ipsearch_results';
		$this->page_title	= 'IPSEARCH_SEARCH_RESULTS';
		$this->user_mode	= ( isset($_POST['user_list']) || isset($_GET['user_list']) ) ? TRUE : FALSE;
		if( $this->user_mode )
		{
			$this->search_user();
		}
		else
		{
			$this->search_ip();
		}
	}

/*	search_ip
	[serip]
		Searches for users by IP address
*/
	function search_ip()
	{
		global $config, $db, $user, $auth, $template, $cache;
		global $phpbb_root_path, $phpbb_admin_path, $phpEx, $table_prefix;
		
		// Doing a search...
		$search_term = request_var('term', '');
		if ( empty($search_term) || strpos($search_term, '.') === FALSE )
		{
			trigger_error($user->lang['IPSEARCH_ENTER_AN_IP'] . adm_back_link($this->u_action));
		}

		$l_host = $host = $ip_list = '';

		// There are three general types of searches.
		//		Search by IP with wildcard
		//		Search by specific full IP
		//		Search by hostname
		// We handle basic setup and getting the search terms for each here.
		if( strpos($search_term, '*') !== FALSE ||
			(strpos($search_term, '_') !== FALSE) && preg_match("/[^a-z]/i", $search_term) )
		{
			// We're doing a search with wildcards.
			$this->use_like		= TRUE;
			$this->lookup_hosts	= TRUE;
			// Search terms are not stored in an array for this type of
			// search. They are on the other types.
			$this->search_terms = str_replace(array('*', '_'), array($db->any_char, $db->one_char), $search_term);
		}
		elseif( preg_match('/([0-9]{1,3}\.){3}[0-9]{1,3}/', $search_term) )
		{
			// Full IP address, yay! Let us try to get the matching host.
			$host = @gethostbyaddr($search_term);
			if( empty($host) || $host == $search_term )
			{
				// Oooo, it failed. Make sure this is an empty string,
				// since PHP handles it differently across versions.
				$host = '';
			}
			$this->search_terms = array($search_term);
		}
		else
		{
			// Searching by a host? Well, okay. IP Fetching time!
			$host				= $search_term;
			$this->search_terms	= @gethostbynamel($host);
			if( !is_array($this->search_terms) )
			{
				// Well, that failed, too.
				trigger_error($user->lang['IPSEARCH_NO_RESOLVE'] . adm_back_link($this->u_action));
			}
		}

		$show_host = $show_ip_list = $no_resolve = FALSE;
		if( !empty($host) && $host != $search_term )
		{
			// They searched for IP and we have a host, so let's show it, too.
			$show_host = TRUE;
			$l_host = sprintf($user->lang['IPSEARCH_HOST'], $host);
		}
		elseif( $host == $search_term )
		{
			// They searched for a host and we resolved this to an IP list.
			// Let's show the resolved IPs.
			$show_ip_list = TRUE;
			$ip_list = sprintf($user->lang['IPSEARCH_IP_LIST'], implode(', ', $this->search_terms));
		}
		elseif( !$show_host && !$show_ip_list )
		{
			$no_resolve = TRUE;
		}

		$template->assign_vars(array(
			'SEARCH_TERM'	=> sprintf($user->lang['IPSEARCH_SEARCHED_FOR'], $search_term),
			'HOST'			=> $l_host,
			'IP_LIST'		=> $ip_list,
			'S_IP_LIST'		=> $show_ip_list,
			'S_HOST'		=> $show_host,
			'S_NO_RESOLVE'	=> $no_resolve,
			'S_AGAIN'		=> TRUE
		));

		$this->define_user_lookups();
		foreach($this->lookups as $v)
		{
			$this->run_query($v);
		}

		$this->search_results['guests'] = array();
		$this->define_guest_lookups();
		foreach($this->guest_lookups as $v)
		{
			$this->run_query($v, true);
		}

		// Now finished with IP lookups. 
		if( !$this->results_found )
		{
			// No search results.
			$message = $user->lang['IP_SEARCH_NADA'] . '<br /><br />';
			$message .= sprintf($user->lang['IPSEARCH_SEARCHED_FOR'], $search_term);
			if( $show_host )
			{
				$message .= '<br /><br />' . $l_host;
			}
			elseif( $show_ip_list )
			{
				$message .= '<br /><br />' . $ip_list;
			}
			elseif( $no_resolve )
			{
				$message .= '<br /><br />' . $user->lang['IPSEARCH_HOST_NOT_FOUND'];
			}
			trigger_error($message . adm_back_link($this->u_action));
		}
		elseif( !empty($this->user_ids) )
		{
			// It is time to get usernames.
			// Clear out duplicate user ids.
			$this->user_ids = array_unique($this->user_ids);

			$result = $db->sql_query('SELECT user_id, username FROM ' . USERS_TABLE . '
						WHERE ' . $db->sql_in_set('user_id', $this->user_ids));
			$users = $db->sql_fetchrowset($result);
			$db->sql_freeresult($result);

			// Print users
			foreach($users as $v)
			{
				$this->ip_output_row($v);
			}
		}

		// Print guests
		$this->ip_output_row(array('user_id' => ANONYMOUS, 'username' => $user->lang['GUEST']));
	}

/*	search_user
	[seruser]
		Displays a list of all the IP addresses associated with a single user.
*/
	function search_user()
	{
		global $db, $user, $template;

		// Get the username and id.
		$username	= request_var('username', '', true);
		$user_id	= request_var('u', 0);
		if (!$user_id)
		{
			$sql = 'SELECT user_id, username
				FROM ' . USERS_TABLE . "
				WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($username)) . "'";
			$result		= $db->sql_query($sql);
			$row		= $db->sql_fetchrow($result);
			$user_id	= (int) $row['user_id'];
			$username	= $row['username'];
			$db->sql_freeresult($result);

			if (!$user_id)
			{
				trigger_error($user->lang['NO_USER'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
		}
		if( $user_id == ANONYMOUS )
		{
			$username = $user->lang['GUEST'];
		}

		// Basic template stuff.
		$template->assign_vars(array(
			'SEARCH_TERM'	=> sprintf($user->lang['IPSEARCH_SEARCHED_FOR'], $username),
			'S_IP_LIST'		=> FALSE,
			'S_HOST'		=> FALSE,
			'S_NO_RESOLVE'	=> FALSE,
			'S_AGAIN'		=> TRUE
		));

		// Get the lookup details and run the search
		$this->search_terms = $user_id;
		$this->define_guest_lookups();
		$this->define_user_lookups();
		$this->lookup_hosts	= TRUE;
		if( $user_id == ANONYMOUS )
		{
			$this->search_results['guests'] = array();
			foreach($this->guest_lookups as $v)
			{
				$this->run_query($v, TRUE);
			}
		}
		else
		{
			foreach($this->lookups as $v)
			{
				$this->run_query($v);
			}
		}

		// Now finished with IP lookups. 
		if( !$this->results_found )
		{
			// No search results.
			$message = $user->lang['IP_SEARCH_NADA'] . '<br /><br />';
			$message .= sprintf($user->lang['IPSEARCH_SEARCHED_FOR'], $username);
			trigger_error($message . adm_back_link($this->u_action));
		}

		// Print results.
		$this->ip_output_row(array('user_id' => $user_id, 'username' => $username));
	}

/*	define_user_lookups
	[dulook]
		Defines the tables and fields searched for IPs of registered users.
*/
	function define_user_lookups()
	{
		/* Search Lookups
			Define the tables and fields we will search for IPs.
			The format is:
		$this->lookups[] = array(
			'cat'		=>	cat name
			'id'		=>	id field,
			'table'		=>	table constant/name,
			'ip'		=>	ip field,
			'no_anon'	=>	true
		);

			Some notes:
			Cat_name is a category name, for internal use in storing results
				related to one area. See the existing entries below for example.
				Do not use the same category in more than one lookup.
			Id field is the field of the table containing the user's id. This
				will be selected and used to look up the user's details in the
				users table later.
			IP field is the field of the table containing IP addresses. This
				field will be searched.
			Table constant/name is the name of the table to be searched.
				Generally is should be a constant defined in constants.php.
			The no_anon line is optional. If included, it will cause the search
				to exclude anonymous users in that table. This is only necessary
				on tables that may hold user and guest ips in the same fields.
				Searches for anonymous users should be done separately.
				If in doubt, go ahead and include the no_anon line.
		*/
		// If you would like to add a search on a table, do it here.


		// Default Lookups for Registered Users
		$this->lookups[] = array(
			'cat'		=>	'posts',
			'id'		=>	'poster_id',
			'table'		=>	POSTS_TABLE,
			'ip'		=>	'poster_ip',
			'no_anon'	=>	true
		);

		$this->lookups[] = array(
			'cat'		=>	'pms',
			'id'		=>	'author_id',
			'table'		=>	PRIVMSGS_TABLE,
			'ip'		=>	'author_ip'
		);

		$this->lookups[] = array(
			'cat'		=>	'sessions',
			'id'		=>	'session_user_id',
			'table'		=>	SESSIONS_TABLE,
			'ip'		=>	'session_ip',
			'no_anon'	=>	true
		);

		$this->lookups[] = array(
			'cat'		=>	'keys',
			'id'		=>	'user_id',
			'table'		=>	SESSIONS_KEYS_TABLE,
			'ip'		=>	'last_ip',
			'no_anon'	=>	true
		);

		$this->lookups[] = array(
			'cat'		=>	'votes',
			'id'		=>	'vote_user_id',
			'table'		=>	POLL_VOTES_TABLE,
			'ip'		=>	'vote_user_ip',
			'no_anon'	=>	true
		);

		$this->lookups[] = array(
			'cat'		=>	'logs',
			'id'		=>	'user_id',
			'table'		=>	LOG_TABLE,
			'ip'		=>	'log_ip',
			'no_anon'	=>	true
		);

		$this->lookups[] = array(
			'cat'		=>	'users',
			'id'		=>	'user_id',
			'table'		=>	USERS_TABLE,
			'ip'		=>	'user_ip',
			'no_anon'	=>	true
		);
	}

/*	define_guest_lookups
	[dglook]
		Defines the tables and fields searched for IPs of guest users.
*/
	function define_guest_lookups()
	{
		/* Search Lookups
			Define the tables and fields we will search for IPs.
			The format of these lookups is mostly the same as that described
			for the registered user lookups above. The differences are:
				We use $this->guest_lookups instead of $this->lookups.
				We may pull details other than the users' id number.
					When pulling extra details, use a comma separated list as
					the value of the id line.  The last item in the list will
					be given the key of "user_id" in the results array.
					Example Usage:
							'id'			=>	'post_id, post_username',
				no_anon has no use and is not needed.
				The extra where line can be used. If present, the value will
					be used to check the user id in the WHERE clause of the
					query. This makes it easier to pull extra columns.
			Be sure to use the same cat names used in the registered user
			searches.
		*/
		$this->guest_lookups[] = array(
			'cat'		=>	'posts',
			'id'		=>	'post_username',
			'table'		=>	POSTS_TABLE,
			'ip'		=>	'poster_ip',
			'where'		=>	'poster_id'
		);

		$this->guest_lookups[] = array(
			'cat'		=>	'sessions',
			'id'		=>	'session_user_id',
			'table'		=>	SESSIONS_TABLE,
			'ip'		=>	'session_ip'
		);

		$this->guest_lookups[] = array(
			'cat'		=>	'votes',
			'id'		=>	'vote_user_id',
			'table'		=>	POLL_VOTES_TABLE,
			'ip'		=>	'vote_user_ip'
		);

		$this->guest_lookups[] = array(
			'cat'		=>	'logs',
			'id'		=>	'user_id',
			'table'		=>	LOG_TABLE,
			'ip'		=>	'log_ip'
		);
	}

/*	prep
	[prep]
       Prepares an array to hold the number of results found by an IP Search.

	Arguments:
	   $user_id - The ID number of the user for which this array will be used.

    Return:
       array  $base  An array, with each value defaulted to 0.
*/
	function prep($user_id)
	{
		global $user;
		$base = array();
		if( $user_id == ANONYMOUS )
		{
			$walk			= $this->guest_lookups;
			$second_walk	= $this->lookups;
		}
		else
		{
			$walk			= $this->lookups;
			$second_walk	= $this->guest_lookups;
		}
		foreach($walk as $v)
		{
			$base[$v['cat']] = 0;
		}
		foreach($second_walk as $v)
		{
			if( !isset($base[$v['cat']]) )
			{
				$base[$v['cat']] = $user->lang['IPSEARCH_NA'];
			}
		}
		return $base;
	}

/*	ip_output_row
	[ipout]
	   Prints out a row of IP results.

	Arguments:
	   $user_info - An array of user details, consisting of at least the
						user_id and username.
*/
	function ip_output_row($user_info)
	{
		global $theme, $phpbb_root_path, $phpEx, $user, $template, $auth;

		$user_id = intval($user_info['user_id']);

		// Make a reference to the result set we're using.
		if( $user_id == ANONYMOUS )
		{
			$base_results =& $this->search_results['guests'];
		}
		else
		{
			$base_results =& $this->search_results;
		}

		if( empty($base_results) )
		{
			return;
		}

		$user_results	= array();
		$totals			= $this->prep($user_id);
		// Run through all results found and extract the ones related to this
		// specific user.
		foreach($base_results as $k=>$v)
		{
			if( $k == 'guests' )
			{
				continue;
			}

			foreach($v as $result)
			{
				if( $user_id != ANONYMOUS && $result['user_id'] != $user_id )
				{
					continue;
				}

				if( !isset($user_results[$result['recorded_ip']]) )
				{
					$user_results[$result['recorded_ip']] = $this->prep($user_id);
				}

				$user_results[$result['recorded_ip']][$k] = $result['total'];
				$totals[$k] += $result['total'];
			}
		}

		// Users/Guests results header
		$heading = ( $user_id != ANONYMOUS ) ? $user->lang['IPSEARCH_REGISTERED'] : $user->lang['G_GUESTS'];
		$show_header = FALSE;
		if( !isset($this->headings[$heading]) )
		{
			$show_header = $this->headings[$heading] = true;
		}

		// If they have permissions to edit users, make the username a link
		// to the ACP edit page
		if( $auth->acl_get('a_user') )
		{
			$user_link = '<a href="' . append_sid("{$phpbb_root_path}adm/index.$phpEx", 'i=users&amp;mode=overview&amp;u=' . $user_id, true, $user->session_id) . '">' . $user_info['username'] . '</a>';
		}
		else
		{
			$user_link = $user_info['username'];
		}

		if( count($user_results) > 1 )
		{
			// More than one IP was found for the user.
			// We need to print out the totals row.
			$tpl_vars = array(
				'USERNAME'			=>	$user_link,
				'SHOW_HEADER'		=>	$show_header,
				'SHOW_TOTAL'		=>	TRUE,
				'HEADER'			=>	$heading,
				'IP'				=>	$user->lang['IPSEARCH_TOTALS'],
			);

			foreach($totals as $cat=>$num)
			{
				$tpl_vars[strtoupper($cat) . '_COUNT'] = $num;
			}
			$template->assign_block_vars('users', $tpl_vars);
			$show_header = FALSE;
		}

		// Print the results for the user
		foreach($user_results as $k=>$v)
		{
			$tpl_vars = array(
				'USERNAME'			=>	$user_link,
				'SHOW_HEADER'		=>	$show_header,
				'SHOW_TOTAL'		=>	FALSE,
				'HEADER'			=>	$heading,
				'IP'				=>	$k,
			);
			if( $this->lookup_hosts )
			{
				if( !array_key_exists($k, $this->found_hosts) )
				{
					$this->found_hosts[$k] = @gethostbyaddr($k);
				}
				if( $this->found_hosts[$k] )
				{
					$tpl_vars['SHOW_LOOKUP']	= TRUE;
					$tpl_vars['LOOKUP']			= $this->found_hosts[$k];
				}
			}
			foreach($v as $cat=>$num)
			{
				$tpl_vars[strtoupper($cat) . '_COUNT'] = $num;
			}
			$template->assign_block_vars('users', $tpl_vars);
			$show_header = FALSE;
		}
	}

/*	run_query
	[query]
       Builds and executes a query to search the database for records related
	   to one or more IP addresses.

    Arguments:
       $details	- Details of a search to be performed.  These are defined above
					as $this->lookups, etc.
       $guests	- Searching for guest users only? TRUE or FALSE.
*/
	function run_query($details, $guests = FALSE)
	{
		global $db;

		$no_anon	= FALSE;
		$cat		= $details['cat'];
		unset($details['cat']);

		$base_query = 'SELECT %1$s as user_id, %3$s as recorded_ip, COUNT(*) as total 
						FROM %2$s
						WHERE %4$s
						GROUP BY %3$s, %1$s';
		if( isset($details['no_anon']) )
		{
			$no_anon = array_pop($details);
		}

		// Build the WHERE section of the query to replace %4$s in the base query.
		$id_field = ( !empty($details['where']) ) ? $details['where']: $details['id'];
		if( $this->user_mode )
		{
			// It's simple for looking up a single user.
			$details['where'] = $id_field . ' = ' . $this->search_terms;
		}
		else
		{
			// Searching by IP is more complex.
			if( $this->use_like )
			{
				// Wildcard search
				$details['where'] = $details['ip'] . ' ' . $db->sql_like_expression($this->search_terms);
			}
			else
			{
				// If not using wildcards, we may have a single IP or a set.
				// We'll use in_set to make it easier.
				$details['where'] = $db->sql_in_set($details['ip'], $this->search_terms);
			}
			if( $no_anon )
			{
				// Exclude guests
				$details['where'] = $id_field . ' <> ' . ANONYMOUS . ' AND ' .  $details['where'];
			}
			elseif( $guests )
			{
				// Include only guests
				$details['where'] = $id_field . ' = ' . ANONYMOUS . ' AND ' .  $details['where'];
			}
		}
		$result = $db->sql_query(vsprintf($base_query, $details));
		while( $row = $db->sql_fetchrow($result) )
		{
			if( !$guests )
			{
				if( !$this->user_mode )
				{
					// Save the user ids for grabbing of usernames later.
					$this->user_ids[]	= $row['user_id'];
				}
				$this->search_results[$cat][]	= $row;
				$this->results_found			= TRUE;
			}
			else
			{
				$this->search_results['guests'][$cat][]	= $row;
				$this->results_found					= TRUE;
			}
		}
		$db->sql_freeresult($result);
	}
}
?>